@@ -37,26 +37,26 @@ import sys
 import os
 import numpy as np
 from phonopy import Phonopy, __version__
-from phonopy.file_IO import (parse_FORCE_CONSTANTS,
-                             parse_FORCE_SETS, write_disp_yaml,
-                             get_born_parameters, write_FORCE_CONSTANTS,
-                             write_force_constants_to_hdf5, parse_QPOINTS,
-                             parse_disp_yaml)
+from phonopy.file_IO import (
+    parse_FORCE_CONSTANTS, parse_FORCE_SETS, get_born_parameters,
+    write_FORCE_CONSTANTS, write_force_constants_to_hdf5, parse_QPOINTS,
+    parse_disp_yaml)
 from phonopy.cui.collect_cell_info import collect_cell_info
 from phonopy.cui.load_helper import read_force_constants_from_hdf5
 from phonopy.cui.settings import PhonopyConfParser
 from phonopy.cui.show_symmetry import check_symmetry
-from phonopy.cui.phonopy_argparse import (get_parser,
-                                          show_deprecated_option_warnings)
+from phonopy.cui.phonopy_argparse import (
+    get_parser, show_deprecated_option_warnings)
 from phonopy.structure.cells import (print_cell, determinant)
 from phonopy.structure.atoms import atom_data, symbol_map
-from phonopy.interface import (create_FORCE_SETS, get_default_physical_units,
-                               get_default_displacement_distance,
-                               write_supercells_with_displacements,
-                               PhonopyYaml, get_interface_mode)
+from phonopy.interface.phonopy_yaml import PhonopyYaml
+from phonopy.interface.calculator import (
+    get_interface_mode, write_supercells_with_displacements,
+    get_default_displacement_distance, get_default_physical_units,
+    create_FORCE_SETS)
 from phonopy.interface.vasp import create_FORCE_CONSTANTS
-from phonopy.phonon.band_structure import (get_band_qpoints,
-                                           get_band_qpoints_by_seekpath)
+from phonopy.phonon.band_structure import (
+    get_band_qpoints, get_band_qpoints_by_seekpath)
 from phonopy.phonon.dos import get_pdos_indices
 
 phonopy_version = __version__
@@ -166,19 +166,18 @@ def finalize_phonopy(log_level,
                      interface_mode,
                      write_displacements=False,
                      filename="phonopy.yaml"):
+    units = get_default_physical_units(interface_mode)
+    phpy_yaml = PhonopyYaml(configuration=phonopy_conf.get_configures(),
+                            physical_units=units,
+                            settings={
+                                'force_sets': False,
+                                'born_effective_charge': False,
+                                'dielectric_constant': False,
+                                'displacements': write_displacements})
+    phpy_yaml.set_phonon_info(phonon)
+    with open(filename, 'w') as w:
+        w.write(str(phpy_yaml))
     if log_level > 0:
-        units = get_default_physical_units(interface_mode)
-        phpy_yaml = PhonopyYaml(configuration=phonopy_conf.get_configures(),
-                                calculator=interface_mode,
-                                physical_units=units,
-                                settings={
-                                    'force_sets': False,
-                                    'born_effective_charge': False,
-                                    'dielectric_constant': False,
-                                    'displacements': write_displacements})
-        phpy_yaml.set_phonon_info(phonon)
-        with open(filename, 'w') as w:
-            w.write(str(phpy_yaml))
         print_end()
     sys.exit(0)
 
@@ -256,6 +255,11 @@ def print_settings(settings,
                 print("  Plus Minus displacement: only one direction")
         if not settings.get_is_diagonal_displacement():
             print("  Diagonal displacement: off")
+        if settings.get_random_displacements():
+            print("  Number of supercells with random displacements: %d"
+                  % settings.get_random_displacements())
+            if settings.get_random_seed() is not None:
+                print("  Random seed: %d" % settings.get_random_seed())
 
     print("Settings:")
     if settings.get_is_nac():
@@ -323,6 +327,40 @@ def print_settings(settings,
             print("    %s" % v)
 
 
+def write_displacements_files(phonon,
+                              phonopy_conf,
+                              log_level,
+                              interface_mode,
+                              optional_structure_info):
+    # From phonopy v1.15.0, displacements are written into phonopy_disp.yaml.
+    # write_disp_yaml(displacements, supercell)
+
+    # Write supercells with displacements
+    cells_with_disps = phonon.get_supercells_with_displacements()
+    N = abs(determinant(phonon.supercell_matrix))
+    write_supercells_with_displacements(interface_mode,
+                                        phonon.supercell,
+                                        cells_with_disps,
+                                        N,
+                                        optional_structure_info)
+
+    if log_level > 0:
+        print("\"phonopy_disp.yaml\" and supercells have been created.")
+        print('')
+        print("NOTE:")
+        print("  Displacements information is written into "
+              "\"phonopy_disp.yaml\" from v1.15.0.")
+        print("  \"disp.yaml\" is still supported to read, but will be "
+              "deprecated.")
+
+    finalize_phonopy(log_level,
+                     phonopy_conf,
+                     phonon,
+                     interface_mode,
+                     write_displacements=True,
+                     filename="phonopy_disp.yaml")
+
+
 #################
 # Option parser #
 #################
@@ -387,8 +425,6 @@ if args.force_sets or args.force_sets_zero:
             dataset = parse_disp_yaml(filename='phonopy_disp.yaml')
             if 'calculator' in dataset:
                 interface_mode = dataset['calculator']
-            else:
-                interface_mode = None
             disp_filename = 'phonopy_disp.yaml'
         except KeyError:
             file_exists('disp.yaml', log_level)
@@ -512,12 +548,13 @@ if run_mode is not None and 'band' in run_mode and is_pdos_auto:
     run_mode = 'band_mesh'
 
 # Prepare phonopy object
-if run_mode == 'displacements':
+if run_mode == 'displacements' and settings.get_temperatures() is None:
     phonon = Phonopy(unitcell,
                      supercell_matrix,
                      primitive_matrix=primitive_matrix,
                      symprec=symprec,
                      is_symmetry=settings.get_is_symmetry(),
+                     calculator=interface_mode,
                      log_level=log_level)
 else:  # Read FORCE_SETS, FORCE_CONSTANTS, or force_constants.hdf5
     # Overwrite frequency unit conversion factor
@@ -537,6 +574,7 @@ else:  # Read FORCE_SETS, FORCE_CONSTANTS, or force_constants.hdf5
         group_velocity_delta_q=settings.get_group_velocity_delta_q(),
         symprec=symprec,
         is_symmetry=settings.get_is_symmetry(),
+        calculator=interface_mode,
         use_lapack_solver=settings.get_lapack_solver(),
         log_level=log_level)
 
@@ -572,12 +610,13 @@ if log_level > 0:
     else:
         print("Use -v option to watch primitive cell, unit cell, "
               "and supercell structures.")
-    print('')
+    if log_level == 1:
+        print("")
 
 #################################
 # Create displacements and exit #
 #################################
-if run_mode == 'displacements':
+if run_mode == 'displacements' and settings.get_temperatures() is None:
     if settings.get_displacement_distance() is None:
         displacement_distance = get_default_displacement_distance(
             interface_mode)
@@ -588,40 +627,19 @@ if run_mode == 'displacements':
         distance=displacement_distance,
         is_plusminus=settings.get_is_plusminus_displacement(),
         is_diagonal=settings.get_is_diagonal_displacement(),
-        is_trigonal=settings.get_is_trigonal_displacement())
-    displacements = phonon.get_displacements()
+        is_trigonal=settings.get_is_trigonal_displacement(),
+        number_of_snapshots=settings.get_random_displacements(),
+        random_seed=settings.get_random_seed())
 
-    # From phonopy v1.15.0, displacements are written into phonopy_disp.yaml.
-    # write_disp_yaml(displacements, supercell)
+    write_displacements_files(phonon,
+                              phonopy_conf,
+                              log_level,
+                              interface_mode,
+                              optional_structure_info)
 
-    # Write supercells with displacements
-    cells_with_disps = phonon.get_supercells_with_displacements()
-    N = abs(determinant(phonon.supercell_matrix))
-    write_supercells_with_displacements(interface_mode,
-                                        supercell,
-                                        cells_with_disps,
-                                        N,
-                                        optional_structure_info)
-
-    if log_level > 0:
-        print("\"phonopy_disp.yaml\" and supercells have been created.")
-        print('')
-        print("NOTE:")
-        print("  Displacements information is written into "
-              "\"phonopy_disp.yaml\" from v1.15.0.")
-        print("  \"disp.yaml\" is still supported to read, but will be "
-              "deprecated.")
-
-    finalize_phonopy(log_level,
-                     phonopy_conf,
-                     phonon,
-                     interface_mode,
-                     write_displacements=True,
-                     filename="phonopy_disp.yaml")
-
-########################################
-# Preparations for phonon calculations #
-########################################
+###################
+# Force constants #
+###################
 if settings.get_read_force_constants():
     if settings.get_is_hdf5() or settings.get_readfc_format() == 'hdf5':
         try:
@@ -669,7 +687,8 @@ else:
         ph_yaml.read(unitcell_filename)
         if (ph_yaml.dataset is not None and
             ('forces' in ph_yaml.dataset or
-             'forces' in ph_yaml.dataset['first_atoms'][0])):
+             ('first_atoms' in ph_yaml.dataset and
+              'forces' in ph_yaml.dataset['first_atoms'][0]))):
             if log_level:
                 print("Forces and displacements are read from \"%s\"."
                       % unitcell_filename)
@@ -688,6 +707,9 @@ else:
         if log_level:
             print("Forces and displacements are read from \"%s\"."
                   % "FORCE_SETS")
+            if 'displacements' in force_sets:
+                print("%d snapshots were found in \"%s\"."
+                      % (len(force_sets['displacements']), "FORCE_SETS"))
 
     if 'natom' in force_sets:
         natom = force_sets['natom']
@@ -703,19 +725,35 @@ else:
             print_error()
         sys.exit(1)
 
+    fc_calculator = None
+    if settings.get_fc_calculator() is not None:
+        from phonopy.interface.fc_calculator import fc_calculator_names
+        if settings.get_fc_calculator().lower() in fc_calculator_names:
+            fc_calculator = settings.get_fc_calculator().lower()
+
+    fc_calculator_options = None
+    if settings.get_fc_calculator_options() is not None:
+        fc_calculator_options = settings.get_fc_calculator_options()
+
     phonon.dataset = force_sets
     if log_level > 0:
-        if settings.get_use_alm():
-            print("Computing force constants by ALM...")
+        if fc_calculator is not None:
+            print("Force constants calculation by %s starts."
+                  % fc_calculator_names[fc_calculator])
         else:
             print("Computing force constants...")
 
     if settings.get_fc_spg_symmetry() or settings.get_is_full_fc():
         # Need to calculate full force constant tensors
-        phonon.produce_force_constants(use_alm=settings.get_use_alm())
+        phonon.produce_force_constants(
+            fc_calculator=fc_calculator,
+            fc_calculator_options=fc_calculator_options)
     else:  # Only force constants between atoms in primitive cell and supercell
-        phonon.produce_force_constants(calculate_full_force_constants=False,
-                                       use_alm=settings.get_use_alm())
+        phonon.produce_force_constants(
+            calculate_full_force_constants=False,
+            fc_calculator=fc_calculator,
+            fc_calculator_options=fc_calculator_options)
+
 
 # Impose cutoff radius on force constants
 cutoff_radius = settings.get_cutoff_radius()
@@ -759,7 +797,9 @@ if settings.get_write_force_constants():
 if settings.get_is_rotational_invariance():
     phonon.get_rotational_condition_of_fc()
 
-# Non-analytical term correction (LO-TO splitting)
+##################################
+# Non-analytical term correction #
+##################################
 if settings.get_is_nac() or has_read_phonopy_yaml:
     def read_nac_params_from_phonopy_yaml(unitcell_filename, log_level):
         ph_yaml = PhonopyYaml()
@@ -818,6 +858,9 @@ if settings.get_is_nac() or has_read_phonopy_yaml:
                     print("%s %12.7f %12.7f %12.7f" % ((text,) + tuple(v)))
             print("-" * 76)
 
+########
+# Misc #
+########
 # Atomic species without mass case
 symbols_with_no_mass = []
 if primitive.get_masses() is None:
@@ -838,7 +881,9 @@ if len(symbols_with_no_mass) > 0:
 # Phonon calculations #
 #######################
 
+#
 # QPOINTS mode
+#
 if run_mode == 'qpoints':
     if settings.get_read_qpoints():
         q_points = parse_QPOINTS()
@@ -866,8 +911,9 @@ if run_mode == 'qpoints':
     else:
         phonon.write_yaml_qpoints_phonon()
 
+#
 # Band structure and mesh sampling
-
+#
 elif (run_mode == 'band' or
       run_mode == 'mesh' or
       run_mode == 'band_mesh' or
@@ -918,7 +964,8 @@ elif (run_mode == 'band' or
         if interface_mode is None:
             comment = None
         else:
-            comment = {'calculator': interface_mode}
+            comment = {'calculator': interface_mode,
+                       'length_unit': physical_units['length_unit']}
 
         if settings.get_is_hdf5() or settings.get_band_format() == 'hdf5':
             phonon.write_hdf5_band_structure(comment=comment)
@@ -1223,8 +1270,9 @@ elif (run_mode == 'band' or
         else:
             plot.show()
 
-
+#
 # Animation
+#
 elif run_mode == 'anime':
     anime_type = settings.get_anime_type()
     if anime_type == "v_sim":
@@ -1254,7 +1302,9 @@ elif run_mode == 'anime':
                 print("band index: %d" % band_index)
                 print("Number of images: %d" % division)
 
+#
 # Modulation
+#
 elif run_mode == 'modulation':
     mod_setting = settings.get_modulation()
     phonon_modes = mod_setting['modulations']
@@ -1273,7 +1323,9 @@ elif run_mode == 'modulation':
     phonon.write_modulations()
     phonon.write_yaml_modulations()
 
+#
 # Ir-representation
+#
 elif run_mode == 'irreps':
     if phonon.set_irreps(settings.get_irreps_q_point(),
                          is_little_cogroup=settings.get_is_little_cogroup(),
@@ -1283,6 +1335,20 @@ elif run_mode == 'irreps':
         phonon.show_irreps(show_irreps)
         phonon.write_yaml_irreps(show_irreps)
 
+#
+# Random displacements at finite temperature
+#
+elif (run_mode == 'displacements' and
+      settings.get_random_displacements() is not None):
+    phonon.generate_displacements(
+        number_of_snapshots=settings.get_random_displacements(),
+        random_seed=settings.get_random_seed(),
+        temperature=settings.get_temperatures()[0])
+    write_displacements_files(phonon,
+                              phonopy_conf,
+                              log_level,
+                              interface_mode,
+                              optional_structure_info)
 else:
     print("-" * 76)
     print(" One of the following run modes may be specified for phonon "
